@@ -1,3 +1,5 @@
+import six
+
 from django.core import serializers
 from django.db import models
 
@@ -83,19 +85,19 @@ def serialize_model(obj, fields=None, include=None, exclude=None,
         return getattr(obj, fieldmap.get(f, f))
 
     if fields is None:
-        fields = fieldmap.keys()
+        fields = list(fieldmap.keys())
 
     if exclude is not None:
         fields = [f for f in fields if f not in exclude]
 
     if include is not None:
         for i in include:
-            if isinstance(i, tuple) or (isinstance(i, basestring)):
+            if isinstance(i, tuple) or (isinstance(i, six.string_types)):
                 fields.append(i)
 
     data = {}
     for f in fields:
-        if isinstance(f, basestring):
+        if isinstance(f, six.string_types):
             data[f] = force_text(getfield(f), strings_only=True)
         elif isinstance(f, tuple):
             k, v = f
@@ -134,7 +136,7 @@ def serialize(src, fields=None, related=None, include=None, exclude=None,
       * a tuple, where the first element is a string key and the second
         is a function taking one argument - function will be run with the
         object being serialized as the argument, and the function result will
-        be included in the result, with the key being the first tuple element)
+        be included in the result, with the key being the first tuple element
 
       * a tuple, where the first element is a related model attribute name
         and the second is a dictionary - related model instance(s) will
@@ -185,13 +187,15 @@ def serialize(src, fields=None, related=None, include=None, exclude=None,
         return serialize(subsrc, fields=fields, include=include,
             exclude=exclude, fixup=fixup)
 
-    if (isinstance(src, models.Manager) or
-            isinstance(src, models.query.QuerySet)):
+    if isinstance(src, models.Manager):
         return [subs(i) for i in src.all()]
 
-    elif isinstance(src, list):
+    elif isinstance(src, list) or isinstance(src, models.query.QuerySet):
         return [subs(i) for i in src]
 
+    elif isinstance(src, dict):
+        return dict((k, subs(v)) for k, v in src.items())
+
     elif isinstance(src, models.Model):
         return serialize_model(src, fields=fields, include=include,
             exclude=exclude, fixup=fixup)
