@@ -4,7 +4,7 @@ from django.views.decorators.csrf import csrf_exempt
 
 from django.conf import settings
 from django.http import HttpResponse
-from .http import Http400, Http200, Http500
+from .http import Http200, Http500, HttpError
 
 import traceback
 import json
@@ -39,26 +39,38 @@ class Endpoint(View):
     then returned.
     """
 
-    @method_decorator(csrf_exempt)
-    def dispatch(self, request, *args, **kwargs):
-        request.content_type = request.META.get('CONTENT_TYPE', 'text/plain')
-        request.params = dict((k, v) for (k, v) in request.GET.items())
-        request.data = None
-        request.raw_data = request.body
-
-        ct = request.content_type.split(";")[0]
-        if ct == 'application/json':
+    @staticmethod
+    def _parse_content_type(content_type):
+        if ';' in content_type:
+            ct, params = content_type.split(';', 1)
             try:
-                if request.body:
-                    request.data = json.loads(request.body)
+                params = dict(param.split('=') for param in params.split())
+            except:
+                params = {}
+        else:
+            ct = content_type
+            params = {}
+        return ct, params
+
+    def _parse_body(self, request):
+        if request.method not in ['POST', 'PUT', 'PATCH']:
+            return
+
+        ct, ct_params = self._parse_content_type(request.content_type)
+        if ct == 'application/json':
+            charset = ct_params.get('charset', 'utf8')
+            try:
+                data = request.body.decode(charset)
+                request.data = json.loads(data)
             except Exception as ex:
-                return Http400('invalid JSON payload: %s' % ex)
+                raise HttpError(400, 'invalid JSON payload: %s' % ex)
         elif ((ct == 'application/x-www-form-urlencoded') or
                 (ct.startswith('multipart/form-data'))):
             request.data = dict((k, v) for (k, v) in request.POST.items())
         else:
             request.data = request.body
 
+    def _process_authenticate(self, request):
         if hasattr(self, 'authenticate') and callable(self.authenticate):
             auth_response = self.authenticate(request)
 
@@ -67,10 +79,23 @@ class Endpoint(View):
             elif auth_response is None:
                 pass
             else:
-                raise TypeError('authenticate method must return HttpResponse instance or None')
+                raise TypeError('authenticate method must return '
+                    'HttpResponse instance or None')
+
+    @method_decorator(csrf_exempt)
+    def dispatch(self, request, *args, **kwargs):
+        request.content_type = request.META.get('CONTENT_TYPE', 'text/plain')
+        request.params = dict((k, v) for (k, v) in request.GET.items())
+        request.data = None
+        request.raw_data = request.body
 
         try:
+            self._parse_body(request)
+            self._process_authenticate(request)
+
             response = super(Endpoint, self).dispatch(request, *args, **kwargs)
+        except HttpError as err:
+            response = err.response
         except Exception as ex:
             if settings.DEBUG:
                 response = Http500(str(ex), traceback=traceback.format_exc())
